<?php

if(!defined('PDO_EASY_LOGS_DIR'))
    define('PDO_EASY_LOGS_DIR', '/var/log/DB_err/');

class PDO_Easy extends PDO
{
    public $debug = false;
    
    public function __construct($dsn = '', $username = '', $password = '', $driver_options = null, $debug = false)
    {
        try {
            parent::__construct($dsn, $username, $password, $driver_options);
	    $this->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
	    $this->debug = $debug;
        } catch (PDOException $e) {
            PDO_Easy::write_log($e->getMessage(), '', $debug);
	    return null;
        }
    }
    
    /**
     *	Działa identycznie jak w PDO, obsługuje automatycznie błędy
     **/
    public function exec($sql)
    {
        try {
            $result = parent::exec($sql);
        } catch (PDOException $e) {
            $this->write_log($e->getMessage(), $sql, $this->debug);
            $result = false;
        }
	return $result;
    }
    
    /**
     *	Uwaga: nie nalezy stosowac do za duzych (ilosc zwracabnych rekordow) zapytan
     *	@param string $sql - zapytanie SELECT
     *	@return array - lista rekordow
     **/
    public function getAll($sql)
    {
        try {
            $sth = $this->prepare($sql);
            $sth->execute();
            $result = $sth->fetchAll(PDO::FETCH_ASSOC);
        } catch (PDOException $e) {
            $this->write_log($e->getMessage(), $sql, $this->debug);
            $result = null;
        }
	return $result;
    }
    
    /**
     *	Uwaga: zapytanie musi posiadac aliasy na pola key i value ktore stana
     *	się kluczami i wartosciami zwracanej tablicy lub tylko aliasy na val,
     *	klucze wygenerują się automatycznie. Kolumna aliasowana na key musi byc
     *	kolumna unikalna lub unikalna w zakresie podawanych danych inaczej moze
     *	dojsc do blednych wynikow
     *	@param string $sql - zapytanie SELECT z aliasami na value i opcjonalnie key
     *	@return array - lista wartosci lub kluczy/wartosci
     **/
    public function getCol($sql)
    {
	$data = $this->getAll($sql);
	$dane = null;
	if(is_array($data) && count($data)>0){
	    if(!isset($data[0]['value'])){
		$this->write_log("Brak aliasu 'value', zle wykorzystanie getCol.", $sql, $this->debug);
		return null;
	    }
	    foreach($data as $dd)
		if(isset($dd['key']))
		    $dane[$dd['key']] = $dd['value'];
		else
		    $dane[] = $dd['value'];
	}
	return $dane;
    }
    
    /**
     *	Funkcja zwraca pojedynczy rekord
     *	@param string $sql - zapytanie SELECT zwracajace jeden wiersz
     *	@return array - pojedynczy rekord
     **/
    public function getRow($sql)
    {
        try {
            $sth = $this->prepare($sql);
            $sth->execute();
            $result = $sth->fetch(PDO::FETCH_ASSOC);
        } catch (PDOException $e) {
            $this->write_log($e->getMessage(), $sql, $this->debug);
            $result = null;
        }
	return $result;
    }
    
    /**
     *	Funkcja zwraca pojedyncze pole
     *	@param string $sql - zapytanie SELECT zwracajace jedno pole
     *	@return mixed - pojedyncze pole
     **/
    public function getOne($sql)
    {
        try {
            $sth = $this->prepare($sql);
            $sth->execute();
            $result = $sth->fetchColumn();
        } catch (PDOException $e) {
            $this->write_log($e->getMessage(), $sql, $this->debug);
            $result = null;
        }
	return $result;
    }
    
    /**
     *	Zapis do logów baz danych
     **/
    public function write_log($error = '', $sql = '', $debug = false)
    {
	if($debug)
            trigger_error('DB error: '.$error.'<br />['.$sql.']');
        $logf = fopen(PDO_EASY_LOGS_DIR.'PDO_EASY_'.date("Y.m.d"), "a");
	if (!strlen($error)){
            $co = implode("\t",array(date("G:i:s"), $error, '['.$sql.']'));
	    fwrite($logf, $co . "\n");
	}
	fclose($logf);
    }
    
    public function isDevel()
    {
	$sql = "SELECT value FROM cms._config WHERE name='base_name'";
	$name = $this->getOne($sql);
	if($name=='DEVEL')
	    return true;
	return false;
    }
    
    public function isWork()
    {
	$sql = "SELECT value FROM cms._config WHERE name='base_name'";
	$name = $this->getOne($sql);
	if($name=='WORK')
	    return true;
	return false;
    }
}

/**
 * uniwersalna funkcja do łączenia z bazą
 * @param string $baza nazwa uchwytu wg definicji z pliku /usr/lib/php/modules/dbcred.php
 * @param string $encoding UTF8 (domyślnie) /LATIN2
 * @return boolean|PDO_Easy object 
 **/
function DB_connect($baza = "burak", $encoding = "UTF8")
{
    include("/usr/lib/php/modules/dbcred.php");
    
    if(!isset($dbcred[$baza]['dbtype']))
	$dbcred[$baza]['dbtype'] = 'pgsql';
    
    $dbcred[$baza]['dsn'] = $dbcred[$baza]['dbtype'] . ":dbname=" . $dbcred[$baza]['dbname'] . ";host=" . $dbcred[$baza]['dbserver'];
    
    if(!is_array($dbcred[$baza])){
	trigger_error('There is no definition for database connection named "'.$baza.'"!', E_USER_WARNING);
	return false;
    }
    
    try {
        $conn = new PDO_Easy($dbcred[$baza]['dsn'], $dbcred[$baza]['dblogin'], $dbcred[$baza]['dbpassword']);
        if($conn && $dbcred[$baza]['dbtype'] == 'pgsql') $conn->exec("SET CLIENT_ENCODING TO $encoding;");
	elseif($conn && $dbcred[$baza]['dbtype'] == 'mysql') $conn->exec("SET NAMES $encoding");
        return $conn;
    } catch (PDOException $e) {
	PDO_Easy::write_log($e->getMessage(), $dbcred[$baza]['dsn']);
	trigger_error('Unable to connect database nemed "'.$baza.'"!', E_USER_WARNING);
        return false;
    }
}

?>