<?php

namespace System\File;
use System\File\NameGeneratorInterface;

class Manager {
	
	const MAX_SIZE_EXCEEDED = 'error_max_file_size_exceeded';
	const TYPE_NOT_SUPPORTED = 'error_file_type_not_supported';
	const UPLOAD_ERROR = 'error_upload_error';
	
	/**
	 * Ścieżka do katalogu z plikami
	 * @var string $path 
	 */
	protected $path;
	
	/** 
     * Lista możliwych do uploadu typów plików
	 * @var array $types 
	 */
	protected $types;
	
	/**
	 * Maksymalny rozmiar pliku w bajtach
	 * @var int $maxSize
	 */
	protected $maxSize;
	
	/**
	 * Maksymalna szerokość pliku jęśli jest obrazkiem
	 * @var int $maxDimensionX
	 */
	protected $maxDimensionX;
	
	/**
	 * Maksymalna wysokość pliku jeśli jest obrazkiem
	 * @var int $maxDimensionY
	 */
	protected $maxDimensionY;
	
	
	/**
	 * W przypadku uploadu plików jest to kopia zmiennej globalnej $_FILES
	 */
	protected $files;
	
	
	/**
	 * Obiekt implementujący interfejs System\File\NameGeneratorInterface
	 * @var System\File\NameGeneratorInterface $nameGenerator
	 */
	protected $nameGenerator;
	
	
	/**
	 * Typ wgrywanego pliku. Opcja umożliwia wykonanie różnych akcji dla różnych typów plików
	 * @var string $type
	 */
	protected $type;
	
	
	/**
	 * Lista parametrów konfiguracyjnych
	 * @var array $cfg
	 */
	protected $cfg;
	
	
	
	protected $contentType;
	protected $contentId;
	
	/**
	 * Konstruktor 
	 * @param array $config zawiera listę konfiguracyjną
	 * Wymagane klucze to:
	 * - path
	 * - type
	 */
	public function __construct( array $cfg ) {
		
		if( !array_key_exists( 'path' , $cfg ) ) {
			throw new \Exception( __METHOD__ . '. "paht" jest wymagany' );
		} if( !array_key_exists( 'type' , $cfg ) ) {
			throw new \Exception( __METHOD__ . '. "type" jest wymagany' );
		}
		
		$this->path = $cfg[ 'path' ];
		$this->type = $cfg[ 'type' ];
		
		if( isset( $cfg[ 'contentType' ] ) ) {
			$this->contentType = (string)$cfg[ 'contentType' ];
		}
		
		if( isset( $cfg[ 'nameGenerator' ] ) ) {
			if( !( $cfg[ 'nameGenerator' ] instanceof NameGeneratorInterface ) ) {
				throw new \Exception( __METHOD__ . '. Wartość klucza nameGenerator musi być obiektem implementującym interfejs "System\File\NameGeneratorInterface"' );
			}
			
			$this->nameGenerator = $cfg[ 'nameGenerator' ];
		}
		
		if( isset( $cfg[ 'types' ] ) ) {
			if( !is_array( $cfg[ 'types' ] ) ) {
				throw new \Exception( __METHOD__ . '. "types" musi być tablica' );
			}
			
			$this->types = $cfg[ 'types' ];
		}
		
		if( isset( $cfg[ 'contentId' ] ) ) {
			$this->contentId = (int)$cfg[ 'contentId' ];
		}
		
		if( isset( $cfg[ 'maxSize' ] ) ) {
			$this->maxSize = (int)$cfg[ 'maxSize' ];
		}
		
		if( isset( $cfg[ 'maxDimensionX' ] ) ) {
			$this->maxDimensionX = (int)$cfg[ 'maxDimensionX'];
		}
		
		if( isset( $cfg[ 'maxDimensionY' ] ) ) {
			$this->maxDimensionY = (int)$cfg[ 'maxDimensionY' ];
		}
		
		$this->cfg = $cfg;
	}	
	
	
	
	
	
	/**
	 * Upload pliku
	 * @param string/array $name nazwa uploadowanego pliku może być stringiem lub tablicą
	 */
	public function upload( $name , &$returnData , $rec = null ) {
		if( is_array( $name ) ) {
			
			$res = array();
			
			foreach( $name as $n ) {
				if( !is_string( $n ) ) {
					throw new \Exception( __METHOD__ . '. Nazwa przesłanego pola musi być stringiem' );
				}
				
				
				if( isset( $res[ 'error' ] ) ) {
					return $res;
				}
				
				$this->upload( $n , null , true );
			}
			
			$returnData = $res;
			return true;
			
		} else {
			
			if( ! ( true === $this->checkSize( $name ) ) ) {
				
				return array(
					'error' => self::MAX_SIZE_EXCEEDED ,
					'content' => round( $this->maxSize / 1024 / 1024 , 2 ) . 'M'
				);
			}
			
			if( ! ( true === $this->checkType( $name ) ) ) {
				
				return array(
					'error' => self::TYPE_NOT_SUPPORTED,
					'content' => $this->types
				);
			}
			
			
			if( 'sitefile' == $this->type ) {
				
				
				if( !is_dir( $this->path ) ) {
					
					return array(
						'error' => self::UPLOAD_ERROR,
						'content' => null
					);
				}
				
				$code = $this->nameGenerator->generateName();
				
				if( false === ( $result = $this->moveFile( $name , $this->contentId , $this->contentType ) ) ) {
					
					return array(
						'error' => self::UPLOAD_ERROR,
						'content' => null
					);
				}
				
				if( null == $rec ) {
					
					$returnData = array_merge( $result , array( 'code' => $code ) );
					return true;
				}
				
				return array_merge( $result , array( 'code' => $code ) );
			} elseif( 'banner' == $this->type ) {
				
				return true;
			}
		}
	}
	
	
	protected function moveFile( $name , $contentId , $contentType ) {
		if( null == ( $file = $this->getFile( $name ) ) ) {
			return false;
		}
		
		if( !is_dir( $this->path . $contentType ) && is_writable( $this->path ) && !mkdir( $this->path . $contentType , 0777 ) ) {
			return false;
		}
		
		$name = $file[ 'name' ];
		$_name = $name;
		$num = 1;
		while( file_exists( $this->path . $contentType . '/' . $_name ) ) {
			
			// zabezpieczenie przed zbyt długim trwanie pętli
			if( $num == 100 ) {
				return false;
			}
			
			
			$__name = explode( '.' , $name );
			$_name = '';
			
			
			for( $i = 0 ; $i < count( $__name ); $i++ ) {
				if( strlen( $_name ) ) {
					$_name .= '.';
				}
				
				if( $i == count( $__name ) - 2 ) {
					
					$_name = $__name[ $i ] . "({$num})";
					
				} else {
					
					$_name .= $__name[ $i ];
				}
			}
			
			$num++;
		}
		
		
		if( !move_uploaded_file( $file[ 'tmp_name' ] , $this->path . $contentType . '/' . $_name ) ) {
			return false;
		}
		
		chmod( $this->path . $contentType . '/' . $_name , 0777 );
		
		
		return array(
			'name' => $_name,
			'path' => $file[ 'tmp_name' ] , $this->path . $contentType . '/',
			'size' => $file[ 'size' ],
			'type' => $file[ 'type' ]
		);
	}
	
	
	/**
	 * Upload plików do nfsa zgodnie z konwencją przyjędą przez poprzednika czyli w katalogu images jest folder o nazwie pierwszych dwuch znaków kodu a w nim plik o nazwie kodu
	 */
	public function uploadEx( $fileName , \System\File\FileDbManagerInterface $dbManager , &$result ) {
		$e = array();
		if( !( $file = $this->getFile( $fileName ) ) ) return false;
		$errors = array();

		if( true === ( $errors = $this->upload( $fileName , $e ) ) ) {
			
			$checksum = md5( file_get_contents( $file[ 'tmp_name' ] ) );
			// sprawdzam sumę kontrolną dla pliku, jeśli tak to zwracam image_name
			if( false !== ( $data = $dbManager->getFileByChecksum( $checksum , true ) ) ) {
				
				$result = true;
				return $data;
			}
			
			// generowanie unikalnego kodu
			$code = $dbManager->generateCode( 12 );
			$dirName = substr( $code , 0 , 2 );
			
			$fileExtension = explode( '.' , $file[ 'name' ] );
			$fileExtension = $fileExtension[ count( $fileExtension ) - 1 ] ;
			
			// upload/pliku
			if( '/' !== substr( $this->path , strlen( $this->path ) , 1) ) {
				$this->path .= '/';
			}
			
			
			// jeśli katalog jeszcze nie istnieje zostaje tu utworzony
			if( !is_dir( $this->path . $dirName ) ) {
				//echo $this->path;
				if( !mkdir( $this->path . $dirName , 0777 ) ) {
					
					throw new \Exception( __METHOD__ . '. Nieudało się utworzyż katalogu' );
				}
			} elseif( !is_writable( $this->path . $dirName ) ) {
				
				chmod( $this->path . $dirName , 0777 );
			}
			
			// przenoszenie pliku z tempa
			if( @!move_uploaded_file( $file[ 'tmp_name' ] , "{$this->path}{$dirName}/{$code}.{$fileExtension}" ) ) {
				
				throw new \Exception( "Nieudało sie Przenieść pliku z tempa = \"{$file[ 'tmp_name' ]}\" do katalofu \"{$this->path}{$dirName}/{$code}.{$fileExtension}\"" );
			}
			
			
			chmod( "{$this->path}{$dirName}/{$code}.{$fileExtension}" , 0755);
			
			$fileDimensions = getimagesize( "{$this->path}{$dirName}/{$code}.{$fileExtension}" );
			
			
			if( !$dbManager->registerFile(
					$code, 
					$fileExtension, 
					$fileDimensions[ 0 ],
					$fileDimensions[ 1 ],
					$file[ 'type' ],
					$checksum		
				)) {
				
				throw new \Exception( "Nieudało sie dodać wpisu z kodem pliku" );
			}
			
			
			
			$result = true;
			return $code;
		}
		
		$result = false;
		return $errors;
	}
	
	public function createFolder( $path ) {
		
	}
	
	protected function isFileExists( $file ) {
		
	}
	
	protected function generateCode( $length = 8 ) {
		
	}
	
	
	// walidacja wielkości pliku
	protected function checkSize( $name ) {
		
		if( null !== $this->maxSize ) {
			
			if( null !== ( $file = $this->getFile( $name ) ) ) {
				
				if( (int)$this->maxSize < (int)$file[ 'size' ] ) {
				
					return $this->maxSize;
				}
				
				return true;
			} 
			
			return false;
		}
		
		return true;
	}
	
	// walidacja typu pliku
	protected function checkType( $name ) {
		
		if( null !== $this->types ) {
			
			if( null !== ( $file = $this->getFile( $name ) ) ) {
				
				if( !in_array( $file[ 'type' ] , $this->types ) ) {
					
					return false;
				}
				
				return true;
			}
			
			return false;
		}
		
		return true;
	}
	
	
	protected function checkDimensions() {
		
		
	}
	
	protected function getFile( $name ) {
		if( null == $this->files ) {
			$this->files = $_FILES;
		}
		
		if( isset( $this->files[ $name ] ) ) {
			return $this->files[ $name ];
		}
		
		return false;
	}
	
	
	public function getFileForCode( $code , $incrementDownloadsCounter = false ) {
	
		
		$Ffu2s = new \Model\Ffu2s;
		$fileData = $Ffu2s->getFile( trim( $code ) );
		
		if( true === $incrementDownloadsCounter && $fileData ) {
			
			$Ffu2s->incrementDownloads( $code );
		}
		
		$contentType = $fileData[ 'type' ];
		
		if( $contentType == 'event' || $contentType == 'publication' ) {
			
			return array(
				'path' => $this->path . $contentType . '/' . $fileData[ 'name' ],
				'name' => $fileData[ 'name' ]
			);
		}
		
	}
	
}

?>