<?php
namespace System\Form;

use Zend\Form\Form;
use Zend\Form\Element as FormElement;
use Zend\Form\ElementInterface;
use Zend\InputFilter\Factory as FilterFactory;
use Zend\Filter;

abstract class AbstractForm {

    const VIEW = 'VIEW';
    const ADD = 'ADD';
    const EDIT = 'EDIT';
    const DELETE = 'DELETE';

    protected $isInit = false;
    protected $form;
    protected $filtersForAll = array();
    protected $filterExceptions = array(); // lista nazw pól, które wyłączone będą z filtrowania przez System\Form\AbstractForm::filterChain
    protected $filterChain = null;
    protected $formMode = null;

    /**
     * Zwraca listę fraz które należy przetłumaczyć i przekazać do konstruktora
     * @return array
     */
    public static function neededTranslations() {
        
    }

    public function __construct(array $translations) {
        
    }

    /**
     * Inicializacja obiektu Zend\Form. Wyamagana jest lista pól
     * @return void
     */
    public final function init($fields) {
        $this->form = new Form($this->name);

        if (is_array($fields)) {

            foreach ($fields as $field) {

                $this->form->add($field);
            }
        }

        $this->isInit = true;
    }

    /**
     * Sprawdza czy tablica przekazana do konstruktora wszystkie przetłumaczone frazy, które dla każedej klasy potomnej są inne. 
     * @return void
     * @throws \Exception w przypadku gdy prynajmnie jeden z kluczy nie zostanie odnaleziony.
     */
    public function parseTranslations($needed) {

        $missingKeys = array();

        foreach ($needed as $pgraseId) {
            if (!array_key_exists($pgraseId, $this->translations)) {
                $missingKeys[] = $pgraseId;
            }
        }

        if (count($missingKeys)) {
            throw new \Exception(__METHOD__ . '. Brak kluczy wymaganych przez konstruktor: ' . implode(',', $missingKeys));
        }
    }

    /**
     * Tworzy obiekt klasy Zend\InputFilter\InputFilter i konfiguruje go zgodnie z zawartością właściwości validation obiektu potomnego.
     * @returns Zemd\InputFilter\InputFilter
     */
    protected function createFilter() {

        $FilterFactory = new FilterFactory();
        $inputFilter = $FilterFactory->createInputFilter($this->validation);

        return $inputFilter;
    }

    /**
     * Walidacja
     * @return bool
     */
    public function isValid($data) {
        $this->setData($data);

        // budowanie filtra i walidatora
        $validator = $this->createFilter($this->validation);
        $this->form->setInputFilter($validator);

        $validationResult = $this->form->isValid();

        // pobranie przefiltrowanych dancyh i podrstawienie ich spowrotem do formularza
        //$this->setData($this->form->getData());

        if ($validationResult) return true;
        return false;
    }

    /**
     * Zwraca obiekt formularza
     * @return Zend\Form\Form
     */
    public function getForm() {

        return $this->form;
    }

    public function setMessages($messages) {

        if (is_array($messages)) {

            $this->form->setMessages($messages);
        }

        return $this;
    }

    public function getMessages() {

        return $this->form->getMessages();
    }

    /**
     * Zamienia klucze błędów przydatne w przypadku przetwymywnia danych w ukrytych polach
     * @param array $inputData regłóły szamiany
     * @returns void
     */
    public function replaceErrors($inputData) {
        $messages = $this->form->getMessages();

        foreach ((array)$inputData as $source => $destination) {

            if (isset($messages[$source])) {

                if (!isset($messages[$destination])) {

                    $messages[$destination] = $messages[$source];
                } else {

                    $messages[$destination] = array_merge($messages[$destination], $messages[$source]);
                }

                unset($messages[$source]);
            }
        }

        $this->form->setMessages($messages);
    }

    /**
     * Adapter dla funkcji Zend\Form\Form::setData(array)
     * @param $data array lista wartości
     * @returns void
     */
    public function setData($data) {

        foreach ($data as $key => $content) {

            if (!in_array($key, $this->filterExceptions) &&
                    $this->form->has($key)) {

                $data[$key] = $this->filter($content);
            } else {

                $data[$key] = $content;
            }
        }

        $this->form->setData($data);
    }

    /**
     * Ustawienie wartości w formularzu
     */
    public function getData() {
        return $this->getForm()->getData();
    }

    /**
     * Wykonanie domyślnych filtrów dla podanej wartości
     * @param mixed $data wartość, która zostanie przefiltrowana
     */
    public function filter($data) {

        $this->initFilter();
        return $this->filterChain->filter($data);
    }

    /**
     * Inicializacji łańcucha filtrów
     */
    protected function initFilter() {
        if (null !== $this->filterChain) {

            return $this->filterChain;
        }

        $this->addachDefaultFilters();

        $this->filterChain = new Filter\FilterChain();

        foreach ($this->filtersForAll as $filter) {

            $this->filterChain->attach($filter);
        }

        return $this->filterChain;
    }

    /**
     * Dodaje filtr i resetuje łaćnuch żeby przy następnym wywołaniu utworzył się na nowo
     * @param Zend\Filter\AbstractFilter $filter filtr...
     */
    public function addachFilter(Filter\AbstractFilter $filter) {
        $this->filtersForAll[] = $filter;
        $this->filterChain = null;
    }

    /**
     * Dodanie domyślnego zestawu filtrów
     */
    public function addachDefaultFilters() {
        if (null !== $this->filterChain) return;

        $this->filtersForAll[] = new Filter\StripTags;
        $this->filtersForAll[] = new Filter\StringTrim;
        $this->filtersForAll[] = new \System\Filter\PregReplaceEx(array('match' => "/'/", 'replace' => "''"));
    }

    /**
     * Pobrnaie błędów przed ich wysąpieniem w celu ukrycia w formularzu i w razie potrzeby odsłonięcia js-em
     * @param string $name nazwa wczytywana z tablicy _messages, która wcześniej musi zostać uzupełniona w konstruktorze najlepiej
     * @returns array tablica treści błędów
     */
    public function getElementMessages($name) {
        if (isset($this->_messages[trim($name)])) {
            return $this->_messages[trim($name)];
        }

        return array();
    }

    /**
     * Dodanie pola do formularza
     * @return void
     */
    public final function addField($field) {
        $this->form->add($field);
    }

    public function setFormMode($formMode) {
        $this->formMode = (string)$formMode;
    }

    public function getFormMode() {
        return (string)$this->formMode;
    }

}

?>