<?php

require_once('access_data/db.conn.php');

/**
 * Rozszerzenie klasy PDO do postaci singletona oraz kilka ułatwiających pisanie funkcji
 * @author Sławek R.
 * @copyright esculap.pl && Slawek R.
 * @version 1.0
 * @final
 * @filesource 
 */
class PDOS extends PDO
{
	/**
	 *	Tablica instancji uchwytów do baz danych.
	 *	@var array
	 */
	protected static $instance = null;
	protected static $last = null;
	protected static $debug = false;
	
	/**
	 *	Konstruktor, można go uruchomić tylko raz, najlepiej kożystać tylko z funkcji getInstance().
	 *	@param string $dbname Nazwa uchwytu bazy danych.
	 *	@uses dbconndata() Aby uzyskac dane uchwytu bazy danych.
	 */
	public function __construct($dbname)
	{
		if(isset(self::$instance[$dbname]))
			trigger_error('Use PDOS::getInstance() function to get single object.', E_USER_ERROR);
		
		$conf = dbconndata($dbname);
		if(!$conf)
			trigger_error('There is no "'.$dbname.'" hendler for DB connection.', E_USER_ERROR);
		$dsn = $conf['dbtype'].':dbname='.$conf['dbname'].';host='.$conf['dbserver'];
		if(isset($conf['dbport'])) $dsn .= ';port='.$conf['dbport'];
		try {
		    parent::__construct($dsn, $conf['dblogin'], $conf['dbpassword']);
		} catch (PDOException $e) {
		    trigger_error('Connection failed: ' . $e->getMessage(), E_USER_ERROR);
		}
		self::$instance[$dbname] = $this;
	}
	
	/**
	 *	Z klonwania nie można korzystać.
	 */
	public function __clone()
	{
		trigger_error('Use getInstance() function to get single object.', E_USER_ERROR);
	}
	
	/**
	 *	Singletonowy zastępnik dla konstruktora, należy go używać do inicjacji obiektu.
	 *	@param string $dbname Nazwa uchwytu bazy danych.
	 *	@return object Instancja obiektu uchwytu do bazy danych.
	 */
	public static function getInstance($dbname = null, $nrlu = false)
	{
	    if(!$dbname){
		    if(self::$last){
		        $dbname = self::$last;
		    }elseif(defined('PDOS_DBNAME')){
		        $dbname = constant('PDOS_DBNAME');
		    }elseif(defined('PORTAL_DB')){//Zgodność ze starymi CFG, wywalić z czasem
		        $dbname = constant('PORTAL_DB');
		    }else{
		        trigger_error("There isn't var dbname, give it to getInstance() or create global constant PDOS_DBNAME!", E_USER_ERROR);
		    }
		}
		
		if(!$nrlu)
		    self::$last = $dbname;
		
		if(defined('PDOS_DEBUG') && PDOS_DEBUG==true)
		    self::$debug = true;
		
		if(!isset(self::$instance[$dbname]))
			self::$instance[$dbname] = new self($dbname);
		return self::$instance[$dbname];
	}
	
	/**
	 * Alias dla getInstance
	 * @param unknown_type $dbname
	 * @param unknown_type $nrlu
	 */
	public static function _($dbname = null, $nrlu = false)
	{
	    return self::getInstance($dbname, $nrlu);
	}
	
    /**
     *	Działa identycznie jak w PDO, dodano jedynie automatyczną obsługę błędów.
     *	@param string $sql Zapytanie do serwera bazy danych.
     *	@return mixed Wyniki zapytania.
     */
    public function exec($sql)
    {
        try {
            $result = parent::exec($sql);
            $arr = $this->errorInfo();
            if($arr[2])
            	$this->handleError($arr[2], $sql);
        } catch (PDOException $e) {
            $this->handleError($e->getMessage(), $sql);
            $result = false;
        }
		return $result;
    }
    
    /**
     *	Uwaga: nie nalezy stosować do za dużych (ilosc zwracabnych rekordów) zapytań.
     *	@param string $sql Zapytanie SELECT.
     *	@return array Tablica rekordów (tablica 2D), tablica tablic asocjacyjnych.
     */
    public function getAll($sql)
    {
        try {
            $sth = $this->prepare($sql);
            $sth->execute();
            $result = $sth->fetchAll(PDO::FETCH_ASSOC);
            $arr = $sth->errorInfo();
            if($arr[2])
            	$this->handleError($arr[2], $sql);
        } catch (PDOException $e) {
            $this->handleError($e->getMessage(), $sql);
            $result = false;
        }
		return $result;
    }
    
    /**
     *	Uwaga: zapytanie musi posiadac aliasy na pola key i value ktore stana
     *	się kluczami i wartosciami zwracanej tablicy lub tylko aliasy na val,
     *	klucze wygenerują się automatycznie. Kolumna aliasowana na key musi byc
     *	kolumna unikalna lub unikalna w zakresie podawanych danych inaczej moze
     *	dojsc do blednych wynikow.
     *	@param string $sql Zapytanie SELECT z aliasami na value i opcjonalnie key.
     *	@return array Tablice w której key to klucz a value to wartość, gdy nie ma key klucze są numeryczne (0,1,2...). 
     */
    public function getCol($sql)
    {
		$data = $this->getAll($sql);
		$dane = null;
		if(is_array($data) && count($data)>0){
		    if(!isset($data[0]['value'])){
				$this->handleError('Query failed: zapytanie musi miec zutowanie jednego pola na wartosc \'value\'!', $sql);
				return false;
		    }
		    foreach($data as $dd)
			if(isset($dd['key'])){
			    $dane[$dd['key']] = $dd['value'];
			}else{
			    $dane[] = $dd['value'];
			}
		}
		return $dane;
    }
    
    /**
     *	Funkcja zwraca pojedynczy rekord.
     *	@param string $sql Zapytanie SELECT zwracajace jeden wiersz.
     *	@return array Jeden rekord, w postaci tablicy asocjacyjnej.
     */
    public function getRow($sql)
    {
        try {
            $sth = $this->prepare($sql);
            $sth->execute();
            $result = $sth->fetch(PDO::FETCH_ASSOC);
        	$arr = $sth->errorInfo();
            if($arr[2])
            	$this->handleError($arr[2], $sql);
        } catch (PDOException $e) {
            $this->handleError($e->getMessage(), $sql);
            $result = false;
        }
		return $result;
    }
    
    /**
     *	Funkcja zwraca pojedyncze pole.
     *	@param string $sql Zapytanie SELECT zwracajace jedno pole lub INSTERT zawierajace RETURNING.
     *	@return mixed - Jedno konkretne pole.
     */
    public function getOne($sql)
    {
        try {
            $sth = $this->prepare($sql);
            $sth->execute();
            $result = $sth->fetchColumn();
        	$arr = $sth->errorInfo();
            if($arr[2])
            	$this->handleError($arr[2], $sql);
        } catch (PDOException $e) {
            $this->handleError($e->getMessage(), $sql);
            $result = false;
        }
		return $result;
    }
    
    /**
     * 
     */
    private function handleError($message, $sql)
    {
        $error_str = null;
        if(self::$debug)
            $error_str = "-------------------BEGIN-------------------\nQuery failed: $message In SQL:\n$sql\n--------------------END--------------------\n";
        else
            $error_str = "Query failed: $message";
        trigger_error($error_str, E_USER_WARNING);
    }
    
    /**
     *	Sprawdza czy podłączony jest do testowej bazy
     */
    public function isDevel()
    {
    	$sql = "SELECT value FROM cms._config WHERE name='base_name'";
    	$name = $this->getOne($sql);
    	if($name=='DEVEL')
    	    return true;
    	return false;
    }
    
    /**
     *	Sprawdza czy podłączony jest do produkcyjnej bazy
     */
    public function isWork()
    {
    	$sql = "SELECT value FROM cms._config WHERE name='base_name'";
    	$name = $this->getOne($sql);
    	if($name=='WORK')
    	    return true;
    	return false;
    }
    
    /**
     * 
     */
    public function debug()
    {
        print_r($this);
    }
}

?>